bin1 <- function(x,ab=nicerange(x),nbin=50,warn=T) {
  n <- length(x)
  if(ab[1]>=ab[2]) stop("Interval vector has negative orientation")
  if(nbin<=0) stop("Number of bin intervals nonpositive")
  r<-.Fortran("bin1",
           as.single(x),
           as.integer(n),
           as.single(ab),
           as.integer(nbin),
           nc=integer(nbin),
           nskip=integer(1))
  if(warn&r$nskip>0) print(paste(r$nskip," points outside the interval ab"))
  list(nc=r$nc,ab=ab,nskip=r$nskip)
}

ash1 <- function(bins,m=5,kopt=c(2,2),warn=T){
  nc <- bins$nc
  ab <- bins$ab
  nbin <- length(nc)
  r <- .Fortran("ash1",
                as.integer(m),
                as.integer(nc),
                as.integer(nbin),
                as.single(ab),
		as.integer(kopt),
                t=single(nbin),
                f=single(nbin),
                single(m),
		ier=integer(1))
  if(warn&r$ier==1) print("ash estimate nonzero outside interval ab")
  list(x=r$t,y=r$f,m=m,ab=ab,kopt=kopt,ier=r$ier)
}

nicerange <- function(x, beta = .1) {
	ab <- range(x)	# interval surrounding data
	del <- ((ab[2] - ab[1]) * beta)/2
	return(c(ab + c( - del, del)))
}

bin2 <- function(x, ab, nbin=c(30,30), warn=T){
  if(missing(ab)){ ab <- t(apply(x,2,nicerange)) }
  n <- nrow(x)
  nbin1 <- nbin[1]
  nbin2 <- nbin[2]
  r <- .Fortran("bin2",
           as.single(x),
           as.integer(n),
           as.single(ab),
           as.integer(nbin),
           as.integer(nbin1),
           as.integer(nbin2),
           nc=integer(nbin[1]*nbin[2]),
           nskip=integer(1))
  if(warn&r$nskip>0) print(paste(r$nskip," points outside the intervals ab"))
  list(nc=matrix(r$nc,nbin[1],nbin[2]),ab=ab,nskip=r$nskip)
}

ash2 <- function(bins,m=c(5,5),kopt=c(2,2),warn=T){
  if(!is.matrix(bins$nc)) stop("bins does not contain bin count matrix")
  if(!is.matrix(bins$ab)) stop("ab not a matrix - should be 2 by 2")
  nc <- bins$nc;  ab <- bins$ab;  nbin <- dim(nc)
  m1 <- m[1]
  m2 <- m[2]
  r <- .Fortran("ash2",
                as.integer(m),
                as.integer(m1),
                as.integer(m2),
                as.integer(nc),
                as.integer(nbin[1]),
                as.integer(nbin[2]),
                as.single(ab),
		as.integer(kopt),
                f=single(nbin[1]*nbin[2]),
                single(m[1]*m[2]),
		ier=integer(1))
  if(warn&r$ier==1) print(" estimate nonzero outside ab rectangle")
  list(z=matrix(r$f,nbin[1],nbin[2]), x=center(ab[1,],nbin[1]),
	y=center(ab[2,],nbin[2]),m=m,ab=ab,kopt=kopt,ier=r$ier)
}

center <- function(ab,k) {	# compute centers of histogram bins
   if(length(ab)!=2) stop("ab not of length 2")
   h <- diff(ab)/k
   seq(ab[1]+h/2,by=h,length=k) 
}

.First.lib <- function(library, section) 
{
   if(version$major == 3) {
     if(version$minor < 2) stop("This library needs S-Plus 3.2 or later")
     else dyn.load(paste(library, section, "sp3", "ash.obj", sep="\\"))
   }
   if(version$major == 4)
     dyn.load(paste(library, section, "sp4", "ash.obj", sep="\\"))
}
